@@ -1,5 +1,5 @@
 # additional polyRAD functions that perform calculations.
-# various intenal functions.
+# various internal functions.
 
 # internal function to take allele frequencies and get prior probs under HWE
 # freqs is a vector of allele frequencies
@@ -182,7 +182,16 @@
                     D = list(c('A', 'K'), c('G', 'W'), c('T', 'R'),
                              c('A', 'D'), c('G', 'D'), c('T', 'D')),
                     B = list(c('C', 'K'), c('G', 'Y'), c('T', 'S'),
-                             c('C', 'B'), c('G', 'B'), c('T', 'B')))
+                             c('C', 'B'), c('G', 'B'), c('T', 'B')),
+                    # throw out deletions with respect to reference
+                    A = list(c('A', '-')), C = list(c('C', '-')),
+                    G = list(c('G', '-')), T = list(c('T', '-')),
+                    # throw out insertions with respect to reference
+                    . = list(c('A', '.'), c('C', '.'), c('G', '.'),
+                             c('T', '.'), c('M', '.'), c('R', '.'),
+                             c('W', '.'), c('S', '.'), c('Y', '.'),
+                             c('K', '.'), c('V', '.'), c('H', '.'),
+                             c('D', '.'), c('B', '.'), c('N', '.')))
   for(i in which(split1 != split2)){
     nucset <- c(split1[i], split2[i])
     splitNew[i] <- "N"
@@ -202,30 +211,31 @@
 
 # substitution matrix for distances between alleles in polyRAD.
 # Any partial match based on ambiguity counts as a complete match.
-polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1, # A
-                          1,0,1,1, 0,1,1,0,0,1, 0,0,1,0, 0,1, # C
-                          1,1,0,1, 1,0,1,0,1,0, 0,1,0,0, 0,1, # G
-                          1,1,1,0, 1,1,0,1,0,0, 1,0,0,0, 0,1, # T
-                          0,0,1,1, 0,0,0,0,0,1, 0,0,0,0, 0,1, # M
-                          0,1,0,1, 0,0,0,0,1,0, 0,0,0,0, 0,1, # R
-                          0,1,1,0, 0,0,0,1,0,0, 0,0,0,0, 0,1, # W
-                          1,0,0,1, 0,0,1,0,0,0, 0,0,0,0, 0,1, # S
-                          1,0,1,0, 0,1,0,0,0,0, 0,0,0,0, 0,1, # Y
-                          1,1,0,0, 1,0,0,0,0,0, 0,0,0,0, 0,1, # K
-                          0,0,0,1, 0,0,0,0,0,0, 0,0,0,0, 0,1, # V
-                          0,0,1,0, 0,0,0,0,0,0, 0,0,0,0, 0,1, # H
-                          0,1,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,1, # D
-                          1,0,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,1, # B
-                          0,0,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,0, # N
-                          1,1,1,1, 1,1,1,1,1,1, 1,1,1,1, 0,0  # -
+polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1,1, # A
+                          1,0,1,1, 0,1,1,0,0,1, 0,0,1,0, 0,1,1, # C
+                          1,1,0,1, 1,0,1,0,1,0, 0,1,0,0, 0,1,1, # G
+                          1,1,1,0, 1,1,0,1,0,0, 1,0,0,0, 0,1,1, # T
+                          0,0,1,1, 0,0,0,0,0,1, 0,0,0,0, 0,1,1, # M
+                          0,1,0,1, 0,0,0,0,1,0, 0,0,0,0, 0,1,1, # R
+                          0,1,1,0, 0,0,0,1,0,0, 0,0,0,0, 0,1,1, # W
+                          1,0,0,1, 0,0,1,0,0,0, 0,0,0,0, 0,1,1, # S
+                          1,0,1,0, 0,1,0,0,0,0, 0,0,0,0, 0,1,1, # Y
+                          1,1,0,0, 1,0,0,0,0,0, 0,0,0,0, 0,1,1, # K
+                          0,0,0,1, 0,0,0,0,0,0, 0,0,0,0, 0,1,1, # V
+                          0,0,1,0, 0,0,0,0,0,0, 0,0,0,0, 0,1,1, # H
+                          0,1,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,1,1, # D
+                          1,0,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,1,1, # B
+                          0,0,0,0, 0,0,0,0,0,0, 0,0,0,0, 0,0,0, # N
+                          1,1,1,1, 1,1,1,1,1,1, 1,1,1,1, 0,0,1, # -
+                          1,1,1,1, 1,1,1,1,1,1, 1,1,1,1, 0,1,0  # .
                           ),
-                         nrow = 16, ncol = 16,
+                         nrow = 17, ncol = 17,
                          dimnames = list(c('A', 'C', 'G', 'T', 'M', 'R', 'W', 
                                            'S', 'Y', 'K', 'V', 'H', 'D', 'B', 
-                                           'N', '-'),
+                                           'N', '-', '.'),
                                          c('A', 'C', 'G', 'T', 'M', 'R', 'W', 
                                            'S', 'Y', 'K', 'V', 'H', 'D', 'B', 
-                                           'N', '-')))
+                                           'N', '-', '.')))
 
 #### Getting genotype priors in mapping pops and simulating selfing ####
 
@@ -261,8 +271,8 @@ polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1, # A
       # add to current gamete set
       nGamCurr <- dim(thisAl)[1]
       nGamNew <- dim(thisIsoGametes)[1]
-      thisAl <- thisAl[rep(1:nGamCurr, each = nGamNew),] + 
-        thisIsoGametes[rep(1:nGamNew, times = nGamCurr),]
+      thisAl <- thisAl[rep(1:nGamCurr, each = nGamNew),, drop = FALSE] + 
+        thisIsoGametes[rep(1:nGamNew, times = nGamCurr),, drop = FALSE]
     }
   } else { # autopolyploids
     thisAl <- sapply(alCopy, function(x){
@@ -274,11 +284,11 @@ polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1, # A
     if(rnd > 1){
       # recursively add alleles to gametes for polyploid
       nReps <- factorial(ploidy-1)/factorial(ploidy - rnd)
-      thisAl <- thisAl[rep(1:ploidy, each = nReps),]
+      thisAl <- thisAl[rep(1:ploidy, each = nReps),, drop = FALSE]
       for(i in 1:ploidy){
         thisAl[((i-1)*nReps+1):(i*nReps),] <- 
-          thisAl[((i-1)*nReps+1):(i*nReps),] + 
-          .makeGametes(alCopy - thisAl[i*nReps,], ploidy - 1, rnd - 1)
+          thisAl[((i-1)*nReps+1):(i*nReps),, drop=FALSE] + 
+          .makeGametes(alCopy - thisAl[i*nReps,, drop = FALSE], ploidy - 1, rnd - 1)
       }
     }
   }
@@ -287,8 +297,15 @@ polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1, # A
 # function to get probability of a gamete with a given allele copy number,
 # given output from makeGametes
 .gameteProb <- function(makeGamOutput, ploidy){
-  return(t(sapply(0:(sum(ploidy)/2), 
-                  function(x) colMeans(makeGamOutput == x))))
+  outmat <- sapply(0:(sum(ploidy)/2), 
+                   function(x) colMeans(makeGamOutput == x))
+  if(ncol(makeGamOutput) == 1){
+    outmat <- matrix(outmat, nrow = ploidy/2 + 1, ncol = 1)
+  } else {
+    outmat <- t(outmat)
+  }
+  
+  return(outmat)
 }
 # function to take two sets of gamete probabilities (from two parents)
 # and output genotype probabilities
@@ -335,3 +352,156 @@ polyRADsubmat <- matrix(c(0,1,1,1, 0,0,0,1,1,1, 0,0,0,1, 0,1, # A
   outmat <- possProgenyProb %*% priors
   return(outmat)
 }
+
+# Functions used by HindHeMapping ####
+
+# Function for making multiallelic gametes.  Autopolyploid segretation only.
+# geno is a vector of length ploidy with values indicating allele identity.
+# rnd is how many alleles will be in the gamete.
+.makeGametes2 <- function(geno, rnd = length(geno)/2){
+  nal <- length(geno) # number of remaining alleles in genotype
+  if(rnd == 1){
+    return(matrix(geno, nrow = nal, ncol = 1))
+  } else {
+    outmat <- matrix(0L, nrow = choose(nal, rnd), ncol = rnd)
+    currrow <- 1
+    for(i in 1:(nal - rnd + 1)){
+      submat <- .makeGametes2(geno[-(1:i)], rnd - 1)
+      theserows <- currrow + 1:nrow(submat) - 1
+      currrow <- currrow + nrow(submat)
+      outmat[theserows,1] <- geno[i]
+      outmat[theserows,2:ncol(outmat)] <- submat
+    }
+    return(outmat)
+  }
+}
+# take output of .makeGametes2 and get progeny probabilities.
+# Return a list, where the first item is a matrix with progeny genotypes in
+# rows, and the second item is a vector of corresponding probabilities.
+.progenyProb2 <- function(gam1, gam2){
+  allprog <- cbind(gam1[rep(1:nrow(gam1), each = nrow(gam2)),],
+                   gam2[rep(1:nrow(gam2), times = nrow(gam1)),])
+  allprog <- t(apply(allprog, 1, sort)) # could be sped up with Rcpp
+  # set up output
+  outprog <- matrix(allprog[1,], nrow = 1, ncol = ncol(allprog))
+  outprob <- 1/nrow(allprog)
+  # tally up unique genotypes
+  for(i in 2:nrow(allprog)){
+    thisgen <- allprog[i,]
+    existsYet <- FALSE
+    for(j in 1:nrow(outprog)){
+      if(identical(thisgen, outprog[j,])){
+        outprob[j] <- outprob[j] + 1/nrow(allprog)
+        existsYet <- TRUE
+        break
+      }
+    }
+    if(!existsYet){
+      outprog <- rbind(outprog, thisgen, deparse.level = 0)
+      outprob <- c(outprob, 1/nrow(allprog))
+    }
+  }
+  return(list(outprog, outprob))
+}
+# Consolidate a list of outputs from .progenyProb2 into a single output.
+# It is assumed the all probabilities in pplist have been multiplied by
+# some factor so they sum to one.
+.consolProgProb <- function(pplist){
+  outprog <- pplist[[1]][[1]]
+  outprob <- pplist[[1]][[2]]
+  if(length(pplist) > 1){
+    for(i in 2:length(pplist)){
+      for(pr1 in 1:nrow(pplist[[i]][[1]])){
+        thisgen <- pplist[[i]][[1]][pr1,]
+        existsYet <- FALSE
+        for(pr2 in 1:nrow(outprog)){
+          if(identical(thisgen, outprog[pr2,])){
+            outprob[pr2] <- outprob[pr2] + pplist[[i]][[2]][pr1]
+            existsYet <- TRUE
+            break
+          }
+        }
+        if(!existsYet){
+          outprog <- rbind(outprog, thisgen, deparse.level = 0)
+          outprob <- c(outprob, pplist[[i]][[2]][pr1])
+        }
+      }
+    }
+  }
+  
+  return(list(outprog, outprob))
+}
+# Probability that, depending on the generation, two alleles sampled in a progeny
+# are different locus copies from the same parent, or from different parents.
+# If there is backcrossing, parent 1 is the recurrent parent.
+# (No double reduction.)
+# Can take a few seconds to run if there are many generations, but it is only
+# intended to be run once for the whole dataset.
+.progAlProbs <- function(ploidy, gen_backcrossing, gen_selfing){
+  # set up parents; number indexes locus copy
+  p1 <- 1:ploidy
+  p2 <- p1 + ploidy
+  # create F1 progeny probabilities
+  progprob <- .progenyProb2(.makeGametes2(p1), .makeGametes2(p2))
+  # backcross
+  if(gen_backcrossing > 0){
+    gam1 <- .makeGametes2(p1)
+    for(g in 1:gen_backcrossing){
+      allprogprobs <- lapply(1:nrow(progprob[[1]]),
+                             function(x){
+                               pp <- .progenyProb2(gam1, .makeGametes2(progprob[[1]][x,]))
+                               pp[[2]] <- pp[[2]] * progprob[[2]][x]
+                               return(pp)
+                             })
+      progprob <- .consolProgProb(allprogprobs)
+    }
+  }
+  # self-fertilize
+  if(gen_selfing > 0){
+    for(g in 1:gen_selfing){
+      allprogprobs <- lapply(1:nrow(progprob[[1]]),
+                             function(x){
+                               gam <- .makeGametes2(progprob[[1]][x,])
+                               pp <- .progenyProb2(gam, gam)
+                               pp[[2]] <- pp[[2]] * progprob[[2]][x]
+                               return(pp)
+                             })
+      progprob <- .consolProgProb(allprogprobs)
+    }
+  }
+  
+  # total probability that (without replacement, from individual progeny):
+  diffp1 <- 0 # two different locus copies, both from parent 1, are sampled
+  diffp2 <- 0 # two different locus copies, both from parent 2, are sampled
+  diff12 <- 0 # locus copies from two different parents are sampled
+  
+  ncombo <- choose(ploidy, 2) # number of ways to choose 2 alleles from a genotype
+  # examine each progeny genotype
+  for(p in 1:nrow(progprob[[1]])){
+    thisgen <- progprob[[1]][p,]
+    thisprob <- progprob[[2]][p]
+    for(m in 1:(ploidy - 1)){
+      al1 <- thisgen[m]
+      for(n in (m + 1):ploidy){
+        al2 <- thisgen[n]
+        if(al1 == al2) next
+        if((al1 %in% p1) && (al2 %in% p1)){
+          diffp1 <- diffp1 + (thisprob / ncombo)
+          next
+        }
+        if((al1 %in% p2) && (al2 %in% p2)){
+          diffp2 <- diffp2 + (thisprob / ncombo)
+          next
+        }
+        if(((al1 %in% p1) && (al2 %in% p2)) ||
+           ((al2 %in% p1) && (al1 %in% p2))){
+          diff12 <- diff12 + (thisprob / ncombo)
+          next
+        }
+        stop("Allele indexing messed up.")
+      }
+    }
+  }
+  
+  return(c(diffp1, diffp2, diff12))
+}
