@@ -1,4 +1,4 @@
-# functions for import of data into RADgen object
+# functions for import of data into RADdata object
 
 # Function to import read counts from UNEAK output (HapMap.hmc.txt).
 # includeloci should be a character vector of loci names to keep in output.
@@ -200,6 +200,7 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
   locTableOut <- data.frame(row.names = as.character(1:nLoc),
                             Chr = character(nLoc),
                             Pos = integer(nLoc),
+                            Ref = character(nLoc),
                             stringsAsFactors = FALSE)
   
   # variables to keep track of which allele and locus we are on
@@ -456,6 +457,7 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
     lastName <- row.names(locTable)[chrset[1]]
     lastPos <- locTable$Pos[chrset[1]]
     lastSeq <- alleleNucleotides[alleles2loc == chrset[1]]
+    lastRef <- locTable$Ref[chrset[1]]
     
     # loop through loci on this chromosome
     for(currLocIn in 2:(length(chrset)+1)){
@@ -465,6 +467,7 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
         thisName <- row.names(locTable)[chrset[currLocIn]]
         thisPos <- locTable$Pos[chrset[currLocIn]]
         thisSeq <- alleleNucleotides[alleles2loc == chrset[currLocIn]]
+        thisRef <- locTable$Ref[chrset[currLocIn]]
         
         # get proportion difference in depth between these two loci
         diff <- sum(abs(rowSums(thisDepth) - rowSums(lastDepth))) / 
@@ -487,6 +490,7 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
         row.names(locTableOut)[currLocOut] <- lastName
         locTableOut$Chr[currLocOut] <- thisChrom
         locTableOut$Pos[currLocOut] <- lastPos
+        locTableOut$Ref[currLocOut] <- lastRef
         
         if(length(chrset) > 1){
           # shift "this" locus to "last" locus
@@ -494,6 +498,7 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
           lastName <- thisName
           lastPos <- thisPos
           lastSeq <- thisSeq
+          lastRef <- thisRef
         }
         
         # increment current allele and locus
@@ -529,8 +534,10 @@ consolidateSNPs <- function(alleleDepth, alleles2loc, locTable, alleleNucleotide
                                                  endPosFromReference),
                                         strand = "+"))[[1]]
           lastSeq <- paste(lastSeq, nonvarSeq, sep = "")
+          lastRef <- paste(lastRef, nonvarSeq, sep = "")
         }
         lastSeq <- paste(lastSeq[alMatch[,1]], thisSeq[alMatch[,2]], sep = "")
+        lastRef <- paste(lastRef, thisRef, sep = "")
         
         # make new depth matrix
 #        print(c(lastName, thisName)) # debug
@@ -669,6 +676,7 @@ VCF2RADdata <- function(file, phaseSNPs = TRUE, tagsize = 80, refgenome = NULL,
   locTable <- data.frame(row.names = as.character(1:expectedLoci),
                          Chr = character(expectedLoci), 
                          Pos = integer(expectedLoci),
+                         Ref = character(expectedLoci),
                          matrix(nrow = expectedLoci, ncol = length(extracols),
                                 dimnames = list(NULL, extracols)),
                          stringsAsFactors = FALSE)
@@ -732,6 +740,7 @@ VCF2RADdata <- function(file, phaseSNPs = TRUE, tagsize = 80, refgenome = NULL,
     thisLocTable <- data.frame(row.names = make.unique(row.names(vcf)),
                                Chr = as.character(SummarizedExperiment::seqnames(vcf)),
                                Pos = Biostrings::start(vcf),
+                               Ref = thisRef,
                                S4Vectors::mcols(vcf)[,extracols],
                                stringsAsFactors = FALSE)
     # set up depth matrix
@@ -898,8 +907,6 @@ VCF2RADdata <- function(file, phaseSNPs = TRUE, tagsize = 80, refgenome = NULL,
   alleles2loc <- alleles2loc[1:currAl]
   alleleNucleotides <- alleleNucleotides[1:currAl]
   locTable <- locTable[1:currLoc, ]
-  # indicate whether non-variable sites included in alleleNucleotides
-  attr(alleleNucleotides, "Variable_sites_only") <- is.null(refgenome)
 
   # build RADdata object
   message("Building RADdata object...")
@@ -910,6 +917,9 @@ VCF2RADdata <- function(file, phaseSNPs = TRUE, tagsize = 80, refgenome = NULL,
                                 min.ind.with.haplotype = min.ind.with.minor.allele)
   radout <- RemoveMonomorphicLoci(radout)
   
+  # indicate whether non-variable sites included in alleleNucleotides
+  attr(radout$alleleNucleotides, "Variable_sites_only") <- is.null(refgenome)
+  
   return(radout)
 }
 
@@ -1055,7 +1065,8 @@ readStacks <- function(allelesFile, matchesFolder, version = 2,
       locTable <- data.frame(row.names = as.character(uniqueLocNames),
                              Chr = tf[[tchrcol]][keeprows],
                              Pos = tf[[tposcol]][keeprows],
-                             Strand = tf[[tstrcol]][keeprows])
+                             Strand = tf[[tstrcol]][keeprows],
+                             stringsAsFactors = FALSE)
     }
     if(version == 2){
       sf <- scan(sumstatsFile, what = sfcols,
@@ -1063,7 +1074,8 @@ readStacks <- function(allelesFile, matchesFolder, version = 2,
       keeprows <- fastmatch::fmatch(uniqueLocNames, sf[[sloccol]])
       locTable <- data.frame(row.names = as.character(uniqueLocNames),
                              Chr = sf[[schrcol]][keeprows],
-                             Pos = sf[[sposcol]][keeprows])
+                             Pos = sf[[sposcol]][keeprows],
+                             stringsAsFactors = FALSE)
     }
   } else {
     # no alignment data
@@ -1225,11 +1237,254 @@ readTASSELGBSv2 <- function(tagtaxadistFile, samFile, min.ind.with.reads = 200,
   
   # build RADdata object
   message("Building RADdata object...")
-  attr(samseq, "Variable_sites_only") <- FALSE
   radout <- RADdata(alleleDepth, alleles2loc, locTable, possiblePloidies,
                     contamRate, samseq)
   radout <- MergeRareHaplotypes(radout, 
                                 min.ind.with.haplotype = min.ind.with.minor.allele)
   radout <- RemoveMonomorphicLoci(radout)
+  attr(radout$alleleNucleotides, "Variable_sites_only") <- FALSE
+  return(radout)
+}
+
+# Function to read output of process_sam_multi.py, so the user can get a
+# preliminary look at Hind/He distribution before going forward with
+# the isolocus_splitter script.
+readProcessSamMulti <- function(alignfile, depthfile = sub("align", "depth", alignfile),
+                                expectedLoci = 1000, min.ind.with.reads = 200,
+                                min.ind.with.minor.allele = 10, possiblePloidies = list(2),
+                                contamRate = 0.001, expectedAlleles = expectedLoci * 15,
+                                maxLoci = expectedLoci){
+  # read file headers
+  aligncon <- file(alignfile, open = 'r')
+  depthcon <- file(depthfile, open = 'r')
+  alignheader <- scan(aligncon, sep = ",", nlines = 1, what = character(),
+                      quiet = TRUE)
+  depthheader <- scan(depthcon, sep = ",", nlines = 1, what = character(),
+                      quiet = TRUE)
+  nalign <- (length(alignheader) - 1) / 3 # number of alignment positions reported
+  samples <- depthheader[-1]
+  nsam <- length(samples)
+  
+  # set up objects to go into RADdata object
+  locTable <- data.frame(Chr = rep(NA_character_, expectedLoci),
+                         Pos = rep(NA_integer_, expectedLoci),
+                         stringsAsFactors = FALSE)
+  locnames <- rep(NA_character_, expectedLoci)
+  alleles2loc <- rep(NA_integer_, expectedAlleles)
+  alleleNucleotides <- rep(NA_character_, expectedAlleles)
+  alleleDepth <- matrix(0L, nrow = nsam, ncol = expectedAlleles,
+                        dimnames = list(samples, NULL))
+  allelenames <- rep(NA_character_, expectedAlleles)
+  
+  # count loci and alleles read in (passing any filtering)
+  loccount <- 0L
+  alcount <- 0L
+  
+  # read the files
+  nscan <- 1e4 # number of lines to read at once
+  whatlistalign <- list(character(), integer(), NULL)
+  whatlistalign <- whatlistalign[c(rep(1, nalign + 1), rep(2, nalign), rep(3, 2 * nalign))]
+  whatlistdepth <- list(character(), integer())
+  whatlistdepth <- whatlistdepth[c(1, rep(2, nsam))]
+  # dummy objects; will hold the last partial marker read
+  lastdepthmat <- matrix(integer(0), nrow = 0, ncol = nsam)
+  lastaligndata <- whatlistalign
+  lastchunk <- FALSE # indicates if we have read the last chunk of the file
+  while(loccount < maxLoci && !lastchunk){
+    aligndata <- scan(aligncon, sep = ",", what = whatlistalign, nlines = nscan,
+                      quiet = TRUE)
+    depthdata <- scan(depthcon, sep = ",", what = whatlistdepth, nlines = nscan,
+                      quiet = TRUE)
+    nread <- length(aligndata[[1]])
+    lastchunk <- nread == 0 # is this the last chunk of the file?
+    stopifnot(nread == length(depthdata[[1]]))
+    if(lastchunk){ # end of file
+      depthmat <- lastdepthmat
+      aligndata <- lastaligndata
+      if(nrow(depthmat) == 0) break
+    } else {
+      depthmat <- matrix(unlist(depthdata[-1]), nrow = nread, ncol = nsam)
+      # merge in any potentially unfinished markers from last chunk
+      depthmat <- rbind(lastdepthmat, depthmat)
+      for(i in 1:(length(whatlistalign) - nalign)){
+        aligndata[[i]] <- c(lastaligndata[[i]], aligndata[[i]])
+      }
+    }
+    nread <- nrow(depthmat) # updated if any loci from last chunk added in
+    if(nread < 2) break # quit if there's nothing left to process
+    # convert number of mutations to matrix for easier processing
+    NM <- matrix(unlist(aligndata[(1:nalign) + (1 + nalign)]),
+                 nrow = nread, ncol = nalign)
+    
+    # find groups of alleles from the same set of alignment positions and process them
+    theseal <- 1L
+    for(i in 2:nread){
+      samegroup <- all(sapply(1:nalign, function(x) aligndata[[x]][i] == aligndata[[x]][i-1]))
+      if(samegroup){
+        theseal <- c(theseal, i)
+      }
+      if(i == nread){
+        lastaligndata <- lapply(aligndata, function(x) x[theseal])
+        lastdepthmat <- depthmat[theseal,]
+      }
+      if(!samegroup || (lastchunk && i == nread)) { # we have one complete group of alleles
+        # divide up into loci based on number of mutations from reference
+        thisNM <- NM[theseal,, drop = FALSE]
+        thisNM <- thisNM[,!is.na(thisNM[1,]), drop = FALSE]
+        hapAssign <- InitHapAssign(thisNM)
+        hapAssign <- lapply(1:nalign, function(x) which(hapAssign == x))
+        for(L in 1:nalign){
+          # skip monomorphic loci or those with no alleles
+          if(length(hapAssign[[L]]) < 2) next
+          # filter by missing data rate
+          thesealSub <- theseal[hapAssign[[L]]]
+          if(sum(colSums(depthmat[thesealSub,]) > 0) < min.ind.with.reads) next
+          # filter by minor allele frequency
+          if(sum(rowSums(depthmat[thesealSub,] > 0) >= min.ind.with.minor.allele) < 2) next
+          # add the locus in if it passed everything
+          thislocname <-aligndata[[L]][thesealSub[1]]
+          firstal <- alcount + 1L
+          alcount <- alcount + length(thesealSub)
+          if(thislocname %in% locnames){ # same locus may show up in multiple groups
+            alleles2loc[firstal:alcount] <- match(thislocname, locnames)
+          } else {
+            loccount <- loccount + 1L
+            locnames[loccount] <- thislocname
+            splitlocname <- strsplit(locnames[loccount], "-")[[1]]
+            locTable$Chr[loccount] <- splitlocname[1]
+            locTable$Pos[loccount] <- as.integer(splitlocname[2])
+            alleles2loc[firstal:alcount] <- loccount
+          }
+          alleleNucleotides[firstal:alcount] <- aligndata[[nalign+1]][thesealSub]
+          allelenames[firstal:alcount] <-
+            paste(locnames[loccount], alleleNucleotides[firstal:alcount], sep = "_")
+          if(alcount > ncol(alleleDepth)){
+            alleleDepth <- alleleDepth[,1:(firstal-1)]
+            alleleDepth <- cbind(alleleDepth, t(depthmat[thesealSub,]))
+          } else {
+            alleleDepth[,firstal:alcount] <- t(depthmat[thesealSub,])
+          }
+          if(loccount >= maxLoci) break
+        }
+      }
+      if(!samegroup){
+        theseal <- i
+      }
+      if(loccount >= maxLoci) break
+    }
+  }
+  
+  # wrap-up and build RADdata object
+  close(aligncon)
+  close(depthcon)
+  
+  locnames <- locnames[1:loccount]
+  locTable <- locTable[1:loccount,]
+  allelenames <- allelenames[1:alcount]
+  alleleNucleotides <- alleleNucleotides[1:alcount]
+  alleleDepth <- alleleDepth[,1:alcount]
+  alleles2loc <- alleles2loc[1:alcount]
+  
+  alleleorder <- order(alleles2loc)
+  allelenames <- allelenames[alleleorder]
+  alleleNucleotides <- alleleNucleotides[alleleorder]
+  alleleDepth <- alleleDepth[,alleleorder]
+  alleles2loc <- alleles2loc[alleleorder]
+  
+  rownames(locTable) <- locnames
+  colnames(alleleDepth) <- allelenames
+  out <- RADdata(alleleDepth, alleles2loc, locTable, possiblePloidies,
+                 contamRate, alleleNucleotides)
+#  out <- MergeRareHaplotypes(out, min.ind.with.haplotype = min.ind.with.minor.allele)
+#  out <- RemoveMonomorphicLoci(out)
+  return(out)
+}
+
+readProcessIsoloci <- function(sortedfile, min.ind.with.reads = 200,
+                               min.ind.with.minor.allele = 10,
+                               min.median.read.depth = 10,
+                               possiblePloidies = list(2), contamRate = 0.001,
+                               nameFromTagStart = TRUE,
+                               mergeRareHap = TRUE){
+  message("Reading file...")
+  incon <- file(sortedfile, open = "r")
+  # read header
+  header <- scan(incon, sep = ",", nlines = 1, what = character(), quiet = TRUE)
+  samples <- header[-(1:5)]
+  nSam <- length(samples)
+  scanwhat <- list(character(), integer(), character(), character(), NULL, integer())
+  scanwhat <- scanwhat[c(1:5, rep(6, nSam))]
+  
+  # read file
+  mydata <- scan(incon, sep = ",", what = scanwhat, quiet = TRUE)
+  close(incon)
+  nAl <- length(mydata[[1]])
+  
+  # get depth matrix
+  alleleDepth <- matrix(unlist(mydata[(1:nSam) + 5]), nrow = nSam, ncol = nAl,
+                         byrow = TRUE, dimnames = list(samples, NULL))
+  if(any(is.na(alleleDepth))){
+    stop("Missing data in depth matrix.")
+  }
+  mydata <- mydata[1:4] # free up space
+  # factor by locus, sorting locus names
+  alleles2loc_factor <- as.factor(mydata[[1]])
+  loci <- levels(alleles2loc_factor)
+  nLoc <- length(loci)
+  alleles2loc <- as.integer(alleles2loc_factor)
+  
+  # perform filtering
+  message("Filtering and sorting loci...")
+  keeploc <- integer(0)
+  for(L in 1:nLoc){
+    submat <- alleleDepth[,alleles2loc == L, drop = FALSE]
+    depthperind <- rowSums(submat)
+    if(sum(depthperind > 0) >= min.ind.with.reads &&
+       sum(colSums(submat > 0) >= min.ind.with.minor.allele) > 1 &&
+       median(depthperind) >= min.median.read.depth){
+      keeploc <- c(keeploc, L)
+    }
+  }
+  keepal <- which(alleles2loc %fin% keeploc)
+  
+  alleles2loc_factor <- droplevels(alleles2loc_factor[keepal])
+  loci <- levels(alleles2loc_factor)
+  alleleDepth <- alleleDepth[,keepal, drop = FALSE]
+  alleles2loc <- as.integer(alleles2loc_factor)
+  alleleNucleotides <- mydata[[3]][keepal]
+  refNucleotides <- mydata[[4]][keepal]
+  colnames(alleleDepth) <- paste(alleles2loc_factor, alleleNucleotides,
+                                 sep = "_")
+  
+  # sort by locus name (i.e. position and chromosome)
+  alorder <- order(alleles2loc)
+  alleleDepth <- alleleDepth[, alorder, drop = FALSE]
+  alleleNucleotides <- alleleNucleotides[alorder]
+  alleles2loc <- alleles2loc[alorder]
+  
+  # build locTable
+  chrom <- sub("\\-.*$", "", loci)
+  loc2al <- fastmatch::fmatch(loci, mydata[[1]])
+  pos <- mydata[[2]][loc2al]
+  ref <- mydata[[4]][loc2al]
+  if(!nameFromTagStart){
+    loci <- paste(chrom, pos, sep = "-")
+  }
+  locTable <- data.frame(row.names = loci,
+                         Chr = chrom, Pos = pos,
+                         Ref = ref,
+                         stringsAsFactors = FALSE)
+  
+  # build RADdata object
+  message("Building RADdata object...")
+  attr(alleleNucleotides, "Variable_sites_only") <- FALSE
+  radout <- RADdata(alleleDepth, alleles2loc, locTable, possiblePloidies,
+                    contamRate, alleleNucleotides)
+  radout <- MergeIdenticalHaplotypes(radout)
+  if(mergeRareHap){
+    radout <- MergeRareHaplotypes(radout, 
+                                  min.ind.with.haplotype = min.ind.with.minor.allele)
+    radout <- RemoveMonomorphicLoci(radout)
+  }
   return(radout)
 }
